{#-------------------------------Category Page---------------------------------#}
{#
Generate the category of page for a entity.
Input:
    topic: properties defining the entity
    properties: all properties of the wiki.
Output: Generates the page Category:<topic.name>
#}
{#-------------------------------Template-------------------------------------#}
__NOTOC__
{{ topic.query_documentation()}}
{{ topic.plural_name }} may be added and edited with the form [[{{ Form.get_page_name(topic) }}]]
* [[{{ ListOf.get_page_name(topic) }}]]
<div class="toccolours mw-collapsible mw-collapsed" style="width:1024px">
{{ topic.name }} description
<div class="mw-collapsible-content">
== uml ==
{% include "uml.jinja" %}

== See also ==
{{ SMWPart.getAllAsPageLink(topic) }}

== Properties ==
{{ SMW.render_as_list(map(Property.get_page_link, Property.get_entity_properties(topic.name, properties))) }}
</div>
</div>

{{ SMW.parser_function("set", oneliner=oneliner) }}